;; Unary frame style, <atc@piffle.force9.co.uk>.     -*- lisp -*-
;;
;; Distributed under the terms of the GNU General Public Licence,
;; version 2, which you should already have several copies of.

(defgroup unary-frame "Unary frame")
(defcustom unary:font default-font
  "Title font for this theme."
  :type font
  :group unary-frame)
(defcustom unary:border-width 3
  "Width of the resize edges."
  :type number
  :range (1 . nil)
  :group unary-frame)
(defcustom unary:fill-colour-desel (get-color "darkslategrey")
  "Border colour."
  :type color
  :group unary-frame)
(defcustom unary:text-colour-desel (get-color "cadetblue4")
  "Detail colour."
  :type color
  :group unary-frame)
(defcustom unary:fill-colour-sel (get-color "seagreen4")
  "Border colour (window with focus)."
  :type color
  :group unary-frame)
(defcustom unary:text-colour-sel (get-color "seagreen3")
  "Detail colour (window with focus)."
  :type color
  :group unary-frame)
(defcustom unary:edge-colour (get-color "black")
  "Surround colour."
  :type color
  :group unary-frame)

(let
    (;; List of customisation names which should have an "':after-set
     ;; rebuild-unary" appended to them.
     (custom-names '(unary:font
		     unary:border-width
		     unary:edge-colour
		     unary:text-colour-sel
		     unary:text-colour-desel
		     unary:fill-colour-sel
		     unary:fill-colour-desel))
     ;; Redefines the style and reframe all windows using it. Kind of
     ;; expensive up front doing it this way, but it allows the frames
     ;; to redraw a tad quicker.
     (rebuild-unary
      (lambda ()
	(let*
	    (;; Colour lists
	     (text-col (list unary:text-colour-desel unary:text-colour-sel))
	     (edge-col unary:edge-colour)
	     (fill-col (list unary:fill-colour-desel unary:fill-colour-sel))

	     ;; Dimension calculations
	     (fill-offset (- 0 unary:border-width))
	     (edge-offset (- -1 unary:border-width))
	     (corner-edge-width (1+ unary:border-width))

	     ;; Tab positioning using 'title-offset	
	     (tab-v-pos (- 1 unary:border-width (font-height unary:font)))
	     (tab-height (- 0 tab-v-pos))
	     (tab-button-width tab-height)
	     (tab-text-width (lambda (w)
			       (+ (/ tab-button-width 2)
				  (text-width (window-name w) unary:font))))
	     (tab-hedge-width (lambda (w)
				(+ tab-button-width 2
				   (tab-text-width w))))
	     (tab-h-pos-min fill-offset)
	     (tab-h-pos-max (lambda (w)
			      (- (car (window-dimensions w))
				 (tab-hedge-width w))))
	     (tab-h-pos (lambda (w)
			  (max tab-h-pos-min
			       (min (+ tab-h-pos-min
				       (or (window-get w 'title-offset) 0))
				    (tab-h-pos-max w)))))
	     (tab-text-offset (lambda (w)
				(+ (tab-h-pos w) tab-button-width)))
	     (tab-vedge-height (+ edge-offset tab-height))
	     (tab-vedge1-offset (lambda (w)
				  (- (tab-h-pos w) 1)))
	     (tab-vedge2-offset (lambda (w)
				  (+ (tab-h-pos w) tab-button-width
				     (tab-text-width w))))
	     (tab-hedge-offset (lambda (w) (- tab-v-pos 1)))
	     ;; Button details
	     (q (/ tab-button-width 6))
	     (r (max 1 (/ tab-button-width 8)))
	     (tab-detail1-v-pos (+ tab-v-pos q))
	     (tab-detail1-size (- tab-button-width (* q 2) 1))
	     (tab-detail1-h-pos (lambda (w) (+ q 1 (tab-h-pos w))))
	     (tab-detail2-v-pos (+ tab-detail1-v-pos r))
	     (tab-detail2-size (- tab-detail1-size (* 2 r)))
	     (tab-detail2-h-pos (lambda (w) (+ r (tab-detail1-h-pos w))))
	     ;; Title updater callback
	     (update-text-width (lambda (w)
				  (if (eq (window-get w 'current-frame-style)
					  'Unary)
				      (rebuild-frame w))))
	     ;; Frame definition fragments	     
	     (f/tab `(;; main text area
		      ((background . ,fill-col)
		       (foreground . ,text-col)
		       (font . ,unary:font)
		       (text . ,window-name)
		       (top-edge . ,tab-v-pos)
		       (left-edge . ,tab-text-offset)
		       (width . ,tab-text-width)
		       (height . ,tab-height)
		       (x-justify . center)
		       (y-justify . center)
		       (class . title))
		      ;; button bg
		      ((background . ,fill-col)
		       (top-edge . ,tab-v-pos)
		       (left-edge . ,tab-h-pos)
		       (width . ,tab-button-width)
		       (height . ,tab-height)
		       (class . title))
		      ;; button details
		      ((background . ,text-col)
		       (top-edge . ,tab-detail1-v-pos)
		       (left-edge . ,tab-detail1-h-pos)
		       (width . ,tab-detail1-size)
		       (height . ,tab-detail1-size)
		       (class . close-button))
		      ((background . ,fill-col)
		       (top-edge . ,tab-detail2-v-pos)
		       (left-edge . ,tab-detail2-h-pos)
		       (width . ,tab-detail2-size)
		       (height . ,tab-detail2-size)
		       (class . close-button))
		      ;; tab edges
		      ((background . ,edge-col)
		       (left-edge . ,tab-vedge1-offset)
		       (top-edge . ,tab-v-pos)
		       (height . ,tab-vedge-height)
		       (width . 1)
		       (class . title))
		      ((background . ,edge-col)
		       (left-edge . ,tab-vedge2-offset)
		       (top-edge . ,tab-v-pos)
		       (height . ,tab-vedge-height)
		       (width . 1)
		       (class . title))
		      ((background . ,edge-col)
		       (left-edge . ,tab-vedge1-offset)
		       (top-edge . ,tab-hedge-offset)
		       (height . 1)
		       (width . ,tab-hedge-width)
		       (class . title)) ))
	     (f/tab-bottom `(((background . ,edge-col)
			      (left-edge . ,tab-vedge1-offset)
			      (top-edge . -1)
			      (height . 1)
			      (width . ,tab-hedge-width)
			      (class . title))
			     ((background . ,edge-col)
			      (left-edge . ,tab-vedge2-offset)
			      (top-edge . ,edge-offset)
			      (height . ,corner-edge-width)
			      (width . 1))
			     ((background . ,edge-col)
			      (left-edge . ,tab-vedge1-offset)
			      (top-edge . ,edge-offset)
			      (height . ,corner-edge-width)
			      (width . 1)) ))
	     (f/borders `(;; top
			  ((background . ,fill-col)
			   (top-edge . ,fill-offset)
			   (height . ,unary:border-width)
			   (left-edge . 0)
			   (right-edge . 0)
			   (class . top-border))
			  ((background . ,edge-col)
			   (top-edge . ,edge-offset)
			   (height . 1)
			   (left-edge . 0)
			   (right-edge . 0)
			   (class . top-border))
			  ;; bottom
			  ((background . ,fill-col)
			   (bottom-edge . ,fill-offset)
			   (height . ,unary:border-width)
			   (left-edge . 0)
			   (right-edge . 0)
			   (class . bottom-border))
			  ((background . ,edge-col)
			   (bottom-edge . ,edge-offset)
			   (height . 1)
			   (left-edge . 0)
			   (right-edge . 0)
			   (class . bottom-border))
			  ;; left
			  ((background . ,fill-col)
			   (left-edge . ,fill-offset)
			   (width . ,unary:border-width)
			   (top-edge . 0)
			   (bottom-edge . 0)
			   (class . left-border))
			  ((background . ,edge-col)
			   (left-edge . ,edge-offset)
			   (width . 1)
			   (top-edge . 0)
			   (bottom-edge . 0)
			   (class . left-border))
			  ;; right
			  ((background . ,fill-col)
			   (right-edge . ,fill-offset)
			   (width . ,unary:border-width)
			   (top-edge . 0)
			   (bottom-edge . 0)
			   (class . right-border))
			  ((background . ,edge-col)
			   (right-edge . ,edge-offset)
			   (width . 1)
			   (top-edge . 0)
			   (bottom-edge . 0)
			   (class . right-border)) ))
	     (f/corners `(;; top/left
			  ((background . ,fill-col)
			   (top-edge . ,fill-offset)
			   (left-edge . ,fill-offset)
			   (height . ,unary:border-width)
			   (width . ,unary:border-width)
			   (class . top-left-corner))
			  ((background . ,edge-col)
			   (top-edge . ,edge-offset)
			   (left-edge . ,edge-offset)
			   (height . ,1)
			   (width . ,corner-edge-width)
			   (class . top-left-corner))
			  ((background . ,edge-col)
			   (top-edge . ,fill-offset)
			   (left-edge . ,edge-offset)
			   (height . ,unary:border-width)
			   (width . 1)
			   (class . top-left-corner))
			  ;; top/right
			  ((background . ,fill-col)
			   (top-edge . ,fill-offset)
			   (right-edge . ,fill-offset)
			   (height . ,unary:border-width)
			   (width . ,unary:border-width)
			   (class . top-right-corner))
			  ((background . ,edge-col)
			   (top-edge . ,edge-offset)
			   (right-edge . ,edge-offset)
			   (height . ,1)
			   (width . ,corner-edge-width)
			   (class . top-right-corner))
			  ((background . ,edge-col)
			   (top-edge . ,fill-offset)
			   (right-edge . ,edge-offset)
			   (height . ,unary:border-width)
			   (width . 1)
			   (class . top-right-corner))
			  ;; bottom/left
			  ((background . ,fill-col)
			   (bottom-edge . ,fill-offset)
			   (left-edge . ,fill-offset)
			   (height . ,unary:border-width)
			   (width . ,unary:border-width)
			   (class . bottom-left-corner))
			  ((background . ,edge-col)
			   (bottom-edge . ,edge-offset)
			   (left-edge . ,edge-offset)
			   (height . ,1)
			   (width . ,corner-edge-width)
			   (class . bottom-left-corner))
			  ((background . ,edge-col)
			   (bottom-edge . ,fill-offset)
			   (left-edge . ,edge-offset)
			   (height . ,unary:border-width)
			   (width . 1)
			   (class . bottom-left-corner))
			  ;; bottom/right
			  ((background . ,fill-col)
			   (bottom-edge . ,fill-offset)
			   (right-edge . ,fill-offset)
			   (height . ,unary:border-width)
			   (width . ,unary:border-width)
			   (class . bottom-right-corner))
			  ((background . ,edge-col)
			   (bottom-edge . ,edge-offset)
			   (right-edge . ,edge-offset)
			   (height . ,1)
			   (width . ,corner-edge-width)
			   (class . bottom-right-corner))
			  ((background . ,edge-col)
			   (bottom-edge . ,fill-offset)
			   (right-edge . ,edge-offset)
			   (height . ,unary:border-width)
			   (width . 1)
			   (class . bottom-right-corner)) ))
	     (normal-frame (append f/borders f/corners f/tab))
	     (border-only-frame (append f/borders f/corners))
	     (shaped-frame (append f/tab f/tab-bottom)) )
	  ;; *pop* - definition of rebuild-unary
	  (add-frame-style 'Unary
			   (lambda (w type)
			     (cond ((eq type 'shaped) shaped-frame)
				   ((eq type 'transient) normal-frame)
				   ((eq type 'shaped-transient) shaped-frame)
				   ((eq type 'unframed) nil-frame)
				   (t normal-frame))))
	  (reframe-windows-with-style 'Unary)
	  (call-after-property-changed 'WM_NAME update-text-width) ))))

  ;; *pop* - outer let
  (rebuild-unary)
  (mapc (lambda (sym)
	  (custom-set-property sym ':after-set rebuild-unary))
	custom-names))
